@@ -613,14 +613,12 @@ public class JSSEngineReferenceImpl extends JSSEngine {
     private void loggingSocketConsumeAllBytes() {
         try {
             int available = s_istream.available();
-            byte[] data = new byte[available];
-            s_istream.read(data);
+            s_istream.skipNBytes(available);
         } catch (Exception e) {}
 
         try {
             int available = c_istream.available();
-            byte[] data = new byte[available];
-            c_istream.read(data);
+            c_istream.skipNBytes(available);
         } catch (Exception e) {}
     }
 
@@ -968,7 +966,7 @@ public class JSSEngineReferenceImpl extends JSSEngine {
             }
 
             debug("JSSEngine: Got inbound alert: " + event);
-
+            event.setEngine(this);
             // Fire inbound alert prior to raising any exception.
             fireAlertReceived(event);
 
@@ -991,7 +989,7 @@ public class JSSEngineReferenceImpl extends JSSEngine {
             }
 
             debug("JSSEngine: Got outbound alert: " + event);
-
+            event.setEngine(this);
             // Fire outbound alert prior to raising any exception. Note that
             // this still triggers after this alert is written to the output
             // wire buffer.
@@ -1262,9 +1260,10 @@ public class JSSEngineReferenceImpl extends JSSEngine {
             }
         } while (this_src_write != 0 || this_dst_write != 0);
 
-        if (seen_exception == false && ssl_exception == null) {
-            ssl_exception = checkSSLAlerts();
-            seen_exception = (ssl_exception != null);
+        SSLException checkException = checkSSLAlerts();
+        if (checkException != null && !seen_exception) {
+            ssl_exception = checkException;
+            seen_exception = true;
         }
 
         // Before we return, check if an exception occurred and throw it if
@@ -1588,6 +1587,11 @@ public class JSSEngineReferenceImpl extends JSSEngine {
         if (is_outbound_closed) {
             debug("Socket is currently closed.");
             handshake_status = SSLEngineResult.Status.CLOSED;
+            if(as_server) {
+                // If is_outbound_closed is true there is no need to wait
+                // for the receipt the peer's close_notify message.
+                closeInbound();
+            }
         }
 
         debug("JSSEngine.wrap() - Finished");
@@ -1699,7 +1703,6 @@ public class JSSEngineReferenceImpl extends JSSEngine {
         cleanup();
     }
 
-
     private class CertValidationTask extends CertAuthHandler {
         public CertValidationTask(SSLFDProxy fd) {
             super(fd);
