@@ -21,13 +21,24 @@ import tempfile
 import argparse
 import textwrap
 
-BLACKLIST = [
+# Exclude new constants added in NSS 3.97 such that this script will
+# create a new PKCS11Constants.java that will match the existing
+# PKCS11Constants.java in base/src/main/java/org/mozilla/jss/pkcs11.
+#
+# https://github.com/dogtagpki/jss/issues/993
+EXCLUDED_CONSTANTS = [
     'CK_PTR',
     'CK_VOID',
     'CK_CALLBACK_FUNCTION',
     'CK_DECLARE_FUNCTION',
     'CK_DECLARE_FUNCTION_POINTER',
-    'CK_UNAVAILABLE_INFORMATION'
+    'CK_UNAVAILABLE_INFORMATION',
+    'CKK_NSS_KYBER',                  # added in NSS 3.97
+    'CKA_NSS_PARAMETER_SET',          # added in NSS 3.97
+    'CKM_NSS_KYBER_KEY_PAIR_GEN',     # added in NSS 3.97
+    'CKM_NSS_KYBER',                  # added in NSS 3.97
+    'CKP_NSS',                        # added in NSS 3.97
+    'CKP_NSS_KYBER_768_ROUND3',       # added in NSS 3.97
 ]
 
 logger = logging.getLogger(__name__)
@@ -465,8 +476,8 @@ def parse_header(header):
         line = file_contents[line_num-1].lstrip()
         if line.startswith('#define'):
             name, value = parse_define(line)
-            if name in BLACKLIST:
-                logger.info("Skipping blacklisted constant: %s", name)
+            if name in EXCLUDED_CONSTANTS:
+                logger.info("Skipping constant: %s", name)
                 continue
 
             logger.info("Found definition %s = %s", name, value)
